@@ -16,9 +16,23 @@
 
 #include "s390_ctr.h"
 
-#define S390_GCM_MAX_TEXT_LENGTH (0x0000000fffffffe0ul) /* (2^31)-32 */
-#define S390_GCM_MAX_AAD_LENGTH  (0x2000000000000000ul) /* (2^61)    */
-#define S390_GCM_MAX_IV_LENGTH   (0x2000000000000000ul) /* (2^61)    */
+/*
+ * NIST SP 800-38d: bitlen(P) <= 2^39 - 256;
+ *   => 0 <= bytelen(P) <= 2^36 - 32
+ */
+#define S390_GCM_MAX_TEXT_LENGTH         ((2ULL << 36) - 32)
+
+/*
+ * NIST SP 800-38d: bitlen(A) <= 2^64 - 1
+ *   => 0 <= bytelen(A) <= 2^61 - 1
+ */
+#define S390_GCM_MAX_AAD_LENGTH          ((2ULL << 61) - 1)
+
+/*
+ * NIST SP 800-38d: 1 <= bitlen(iv) <= 2^64 - 1
+ *   => 1 <= bytelen(iv) <= 2^61 - 1
+ */
+#define S390_GCM_MAX_IV_LENGTH           ((2ULL << 61) - 1)
 
 /* the recommended iv length for GCM is 96 bit or 12 byte */
 #define GCM_RECOMMENDED_IV_LENGTH 12
@@ -51,8 +65,8 @@ struct kma_ctx_t {
 	uint32_t cv;
 	ica_aes_vector_t tag;
 	ica_aes_vector_t subkey_h;
-	uint64_t total_aad_length;
-	uint64_t total_input_length;
+	uint64_t total_aad_length; /* bit length */
+	uint64_t total_input_length; /* bit length */
 	ica_aes_vector_t j0;
 	ica_aes_key_len_256_t key;
 	// Above this line: KMA parmblock, never change!
@@ -735,8 +749,8 @@ static inline int s390_aes_gcm_kma(const unsigned char *in_data,
 		data_length = 0;
 
 	/* Actual lengths needed by KMA */
-	ctx->total_aad_length += aad_length*8;
-	ctx->total_input_length += data_length*8;
+	ctx->total_aad_length += aad_length * 8;
+	ctx->total_input_length += data_length * 8;
 
 	/* Call KMA */
 	rc = s390_kma(hw_fc, ctx,
