@@ -12,6 +12,8 @@
 #include <stdbool.h>
 #include <stddef.h>
 
+#include "s390_ecc.h"
+
 #define AES128_KEYLEN	(128 / 8)
 #define AES192_KEYLEN	(192 / 8)
 #define AES256_KEYLEN	(256 / 8)
@@ -125,62 +127,28 @@ struct aes_cmac_tv {
 	unsigned int rv;
 };
 
-struct des3_ecb_tv {
-	size_t len;
-	unsigned char key[DES3_KEYLEN];
-	unsigned char *plaintext;
-	unsigned char *ciphertext;
+struct ecdsa_kat_tv {
+	unsigned int nid;
+	unsigned char *x;
+	unsigned char *y;
+	unsigned char *k;
+	unsigned char *d;
+	unsigned char *hash;
+	unsigned int hashlen;
+	unsigned char *sig;
+	unsigned int siglen;
 };
 
-struct des3_cbc_tv {
-	size_t len;
-	unsigned char key[DES3_KEYLEN];
-	unsigned char iv[DES3_BLKSIZE];
-	unsigned char *plaintext;
-	unsigned char *ciphertext;
-};
-
-struct des3_cbc_cs_tv {
-	size_t len;
-	unsigned char key[DES3_KEYLEN];
-	unsigned char iv[DES3_BLKSIZE];
-	unsigned char *plaintext;
-	unsigned char *ciphertext;
-	int variant;
-};
-
-struct des3_cfb_tv {
-	size_t len;
-	int lcfb;
-	unsigned char key[DES3_KEYLEN];
-	unsigned char iv[DES3_BLKSIZE];
-	unsigned char *plaintext;
-	unsigned char *ciphertext;
-};
-
-struct des3_ofb_tv {
-	size_t len;
-	unsigned char key[DES3_KEYLEN];
-	unsigned char iv[DES3_BLKSIZE];
-	unsigned char *plaintext;
-	unsigned char *ciphertext;
-};
-
-struct des3_ctr_tv {
-	size_t len;
-	unsigned char key[DES3_KEYLEN];
-	unsigned char ctr[DES3_BLKSIZE];
-	unsigned char *plaintext;
-	unsigned char *ciphertext;
-};
-
-struct des3_cmac_tv {
-	size_t msglen;
-	size_t maclen;
-	unsigned char key[DES3_KEYLEN];
-	unsigned char *msg;
-	unsigned char *mac;
-	unsigned int rv;
+struct ecdh_kat_tv {
+	unsigned int nid;
+	unsigned int privlen;
+	unsigned char *da;
+	unsigned char *xa;
+	unsigned char *ya;
+	unsigned char *db;
+	unsigned char *xb;
+	unsigned char *yb;
+	unsigned char *z;
 };
 
 struct rsa_tv {
@@ -234,6 +202,67 @@ struct drbg_sha512_tv {
 	unsigned char *prnd;
 };
 
+struct ecdsa_tv {
+	/* sign inputs */
+	const ICA_EC_KEY *key;
+	int hash;
+	unsigned char *msg; /* should be qualified const,
+			        but sha api lacks const ... */
+	size_t msglen;
+	const unsigned char *k;
+	/* sign expected outputs */
+	const unsigned char *r;
+	const unsigned char *s;
+	size_t siglen;
+};
+
+struct scalar_mul_tv {
+	/* scalar mul inputs */
+	int curve_nid;
+	size_t len;
+	const unsigned char *scalar;
+
+	/* scalar mul outputs */
+	const unsigned char *x;
+	const unsigned char *y;
+};
+
+struct scalar_mulx_tv {
+	/* scalar mul inputs */
+	int curve_nid;
+	size_t len;
+	const unsigned char *scalar;
+	const unsigned char *u;
+
+	/* scalar mul outputs */
+	const unsigned char *res_u;
+};
+
+struct scalar_mulx_it_tv {
+	/* scalar mul inputs */
+	int curve_nid;
+	size_t len;
+	const unsigned char *scalar_u;
+
+	/* scalar mul outputs */
+	const unsigned char *res_u_it1;
+	const unsigned char *res_u_it1000;
+	const unsigned char *res_u_it1000000;
+};
+
+struct scalar_mulx_kex_tv {
+	/* scalar mul inputs */
+	int curve_nid;
+	size_t len;
+	const unsigned char *a_priv;
+	const unsigned char *b_priv;
+
+	/* scalar mul outputs */
+	const unsigned char *a_pub;
+	const unsigned char *b_pub;
+	const unsigned char *shared_secret;
+};
+
 #ifdef ICA_FIPS
 extern const struct aes_ecb_tv AES_ECB_TV[];
 extern const size_t AES_ECB_TV_LEN;
@@ -265,30 +294,15 @@ extern const size_t AES_XTS_TV_LEN;
 extern const struct aes_cmac_tv AES_CMAC_TV[];
 extern const size_t AES_CMAC_TV_LEN;
 
-extern const struct des3_ecb_tv DES3_ECB_TV[];
-extern const size_t DES3_ECB_TV_LEN;
-
-extern const struct des3_cbc_tv DES3_CBC_TV[];
-extern const size_t DES3_CBC_TV_LEN;
-
-extern const struct des3_cbc_cs_tv DES3_CBC_CS_TV[];
-extern const size_t DES3_CBC_CS_TV_LEN;
-
-extern const struct des3_cfb_tv DES3_CFB_TV[];
-extern const size_t DES3_CFB_TV_LEN;
-
-extern const struct des3_ofb_tv DES3_OFB_TV[];
-extern const size_t DES3_OFB_TV_LEN;
-
-extern const struct des3_ctr_tv DES3_CTR_TV[];
-extern const size_t DES3_CTR_TV_LEN;
-
-extern const struct des3_cmac_tv DES3_CMAC_TV[];
-extern const size_t DES3_CMAC_TV_LEN;
-
 extern const struct rsa_tv RSA_TV[];
 extern const size_t RSA_TV_LEN;
 
+extern const struct ecdsa_kat_tv ECDSA_KAT_TV[];
+extern const size_t ECDSA_KAT_TV_LEN;
+
+extern const struct ecdh_kat_tv ECDH_KAT_TV[];
+extern const size_t ECDH_KAT_TV_LEN;
+
 extern const struct sha_tv SHA1_TV[];
 extern const size_t SHA1_TV_LEN;
 
@@ -303,8 +317,39 @@ extern const size_t SHA384_TV_LEN;
 
 extern const struct sha_tv SHA512_TV[];
 extern const size_t SHA512_TV_LEN;
+
+extern const struct sha_tv SHA3_224_TV[];
+extern const size_t SHA3_224_TV_LEN;
+
+extern const struct sha_tv SHA3_256_TV[];
+extern const size_t SHA3_256_TV_LEN;
+
+extern const struct sha_tv SHA3_384_TV[];
+extern const size_t SHA3_384_TV_LEN;
+
+extern const struct sha_tv SHA3_512_TV[];
+extern const size_t SHA3_512_TV_LEN;
+
 #endif /* ICA_FIPS */
 
+#ifdef ICA_INTERNAL_TEST_EC
+extern const struct ecdsa_tv ECDSA_TV[];
+extern const size_t ECDSA_TV_LEN;
+
+extern const struct scalar_mul_tv SCALAR_MUL_TV[];
+extern const size_t SCALAR_MUL_TV_LEN;
+
+extern const struct scalar_mulx_tv SCALAR_MULX_TV[];
+extern const size_t SCALAR_MULX_TV_LEN;
+
+extern const struct scalar_mulx_it_tv SCALAR_MULX_IT_TV[];
+extern const size_t SCALAR_MULX_IT_TV_LEN;
+
+extern const struct scalar_mulx_kex_tv SCALAR_MULX_KEX_TV[];
+extern const size_t SCALAR_MULX_KEX_TV_LEN;
+
+#endif /* ICA_INTERNAL_TEST_EC */
+
 extern const struct drbg_sha512_tv DRBG_SHA512_TV[];
 extern const size_t DRBG_SHA512_TV_LEN;
 
