@@ -149,9 +149,7 @@ static int read_cpuinfo(void)
 	FILE *handle = fopen("/proc/cpuinfo", "r");
 	if (handle) {
 		char buffer[80];
-		int i = 0;
 		while(fgets(buffer, sizeof(buffer), handle)) {
-			i++;
 			if(strstr(buffer,"features") && strstr(buffer,"msa")) {
 				msa = 1;
 				break;
@@ -273,10 +271,16 @@ static void set_switches(int msa)
 		if (S390_CRYPTO_TEST_MASK(mask, s390_kimd_functions[n].hw_fc))
 			*s390_kimd_functions[n].enabled = 1;
 
+	/* Turn sha3/shake availability off if MSA8 is not available. */
+	if (msa < 8) {
+		for (n = SHA_3_224; n < SHAKE_256; n++)
+			*s390_kimd_functions[n].enabled = 0;
+	}
+
 	/* pcc query */
 	memset(mask, 0, sizeof(mask));
-	if (4 <= msa) {
-		msa4_switch = 1;
+	if (9 <= msa) {
+		msa9_switch = 1;
 		if (begin_sigill_section(&oldact, &oldset) == 0)
 			s390_pcc(S390_CRYPTO_QUERY, mask);
 		end_sigill_section(&oldact, &oldset);
